import math
from flask import Flask, render_template, url_for, jsonify, request
from div import RosComunication
from threading import Lock, Thread
from div import randomData
from div.ClientUdpSocket import ClientUdpSocket
from div.ClientTcpSocket import ClientTcpSocket
from div import Parser
import cv2
import json
import numpy as np
import os

debug = False
rubenIp = '192.168.0.103'
localHost = "127.0.0.1"
serverIp = rubenIp

lidarPort = 5005
vesselInfoDataPort = 5006
mapPort = 5007

rosComuncationPort = 5015


UDPLidarSocket = ClientUdpSocket(serverIp, lidarPort, debug=False)
lidarSocketThread = Thread(target=UDPLidarSocket.run)
lidarSocketThread.start()

UDPVesselInfoSocket = ClientUdpSocket(serverIp, vesselInfoDataPort, debug=False)
vesselSocketTread = Thread(target= UDPVesselInfoSocket.run)
vesselSocketTread.start()

UDPMapSocket = ClientUdpSocket(serverIp, mapPort, debug=False)
mapSocketThread = Thread(target=UDPMapSocket.run)
mapSocketThread.start()

TCPRosComunicationSocket = ClientTcpSocket(serverIp, rosComuncationPort, debug=False)
rosComuncationThread = Thread(target=TCPRosComunicationSocket.run)
rosComuncationThread.start()

app = Flask(__name__)
@app.route('/', methods=["GET", "POST"])

def homepage():
    """
    sends the homepage for the GUI
    :return: returns the rendered template for the homepage of the GUI
    """
    if request.method == "POST":
        if "Emergency_btn" in request.form:
            print(RosComunication.sendEMERGENCYCommand())
            data = json.dumps(RosComunication.sendEMERGENCYCommand())
            TCPRosComunicationSocket.sendData(data)

        elif "Start_btn" in request.form:
            print(RosComunication.sendStartCommand())
            data = json.dumps(RosComunication.sendStartCommand())
            TCPRosComunicationSocket.sendData(data)

        elif "Stop_btn" in request.form:
            print(RosComunication.sendStopCommand())
            data = json.dumps(RosComunication.sendStopCommand())
            TCPRosComunicationSocket.sendData(data)

        elif "Auto_btn" in request.form:
            print(RosComunication.sendAutonomousCommand())
            data = json.dumps(RosComunication.sendAutonomousCommand())
            TCPRosComunicationSocket.sendData(data)

        elif "Manually_btn" in request.form:
            print(RosComunication.sendManuallyCommand())
            data = json.dumps(RosComunication.sendManuallyCommand())
            TCPRosComunicationSocket.sendData(data)

    return render_template("index.html")


@app.route("/nav/", methods=["GET", "POST"])
def naviPage():
    """
    sends the navigation page to the GUI.

    :return: returns the navigation page to the GUI
    """
    lidarData = [[0] * 360, [0] * 360]
    motorSpeed1 = 0
    motorspeed2 = 0
    motorSpeed3 = 0
    motorSpeed4 = 0
    vesselSpeedX = 0
    vesselSpeedY = 0
    vesselSpeedTheta = 0

    if request.method == "POST":
        if "Emergency_btn" in request.form:
            print(RosComunication.sendStopCommand())
            data = json.dumps(RosComunication.sendEMERGENCYCommand())
            TCPRosComunicationSocket.sendData(data)
    return render_template("navi.html",
                           lidarData=lidarData,
                           motorSpeed1=motorSpeed1,
                           motorspeed2=motorspeed2,
                           motorSpeed3=motorSpeed3,
                           motorSpeed4=motorSpeed4,
                           vesselSpeedX = vesselSpeedX,
                           vesselSpeedY = vesselSpeedY,
                           vesselSpeedTheta = vesselSpeedTheta)


@app.route("/nav/update/", methods=["GET", "POST"])
def updateNaviPage():
    """
    sends the updated parameters to the GUI Navigation Page
    :return: returns a json object containing the updated parameters.
    """
    if request.method == "POST":
        if "Emergency_btn" in request.form:
            print(RosComunication.sendEMERGENCYCommand())
            data = json.dumps(RosComunication.sendEMERGENCYCommand())
            TCPRosComunicationSocket.sendData(data)

############_Retrive_data_and_compute_it_#########
    lidarUdpData = UDPLidarSocket.retriveData()
    if debug == True:
        print("lidar Data:")
        print(lidarUdpData)
    lidarData = Parser.lidarDataParser(lidarUdpData)

    vesselUdpData = UDPVesselInfoSocket.retriveData()
    if debug == True:
        print("vessel Data:")
        print(vesselUdpData)
    vesselData = Parser.vesselInfoDataParser(vesselUdpData)

    mapUdpData = UDPMapSocket.retriveData()
    if debug == True:
        print("map Data:")
        print(mapUdpData)
    image = Parser.mapDataParser(mapUdpData)
    appRoot = os.path.dirname(os.path.abspath(__file__))
    target = os.path.join(appRoot, "static\\map.jpg")
    if image is not None:
        if debug == True:
            print("URL path for image: ")
            print(target)
        cv2.imwrite(target, image)

################_RANDOM_Data_Test################
    # lidarData = randomData.getRandomLidarData(12)
    # motorSpeed1 = randomData.getRandomSpeedData(10)
    # motorSpeed2 = randomData.getRandomSpeedData(10)
    # motorSpeed3 = randomData.getRandomSpeedData(10)
    # motorSpeed4 = randomData.getRandomSpeedData(10)
    #
    # vesselSpeedX = randomData.getRandomSpeedData(10)
    # vesselSpeedY = randomData.getRandomSpeedData(10)
    # vesselSpeedTheta = randomData.getRandomSpeedData(10)
#################################################

    return jsonify(points=lidarData[0][::-1],
                   angle=lidarData[1][::-1],
                   motor1=vesselData[0],
                   motor2=vesselData[1],
                   motor3=vesselData[2],
                   motor4=vesselData[3],
                   vesselSpeedX = vesselData[4],
                   vesselSpeedY = vesselData[5],
                   vesselSpeedTheta = vesselData[6])
                   #need to add update for vessel speed in x,y and theta direction

@app.route("/adv/", methods = ["GET", "POST"])
def advPage():
    """
    sends the advanced page to the GUI.
    :return: returns the rendered template to the GUI.
    """
    if request.method == "POST":
        if "Emergency_btn" in request.form:
            print(RosComunication.sendEMERGENCYCommand())
            data = json.dumps(RosComunication.sendEMERGENCYCommand())
            TCPRosComunicationSocket.sendData(data)

    return render_template("advanced.html")

@app.route("/adv/update/", methods =["GET","POST"])
def updateAdvPage():
    """
    sends the updated parameters to the GUI advanced page.
    :return: returns a json object containing the updated parameters.
    """
    if request.method == "POST":
        if "Emergency_btn" in request.form:
            print(RosComunication.sendEMERGENCYCommand())
            data = json.dumps(RosComunication.sendEMERGENCYCommand())
            TCPRosComunicationSocket.sendData(data)

    return jsonify()


if __name__ == "__main__":
    app.run(static_url_path= "/static", Debug=True)
