

import socket
import time
from threading import Lock, Thread


class ClientTcpSocket:


    def __init__(self, ip, port, debug=False):
        """
        creats a TCP socket object

        :param ip: desierd ip to connect to
        :param port: type of port to connect to
        :param debug: set to true if you want do debugg
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.ip = ip
        self.port = port
        self.debug = debug
        self.data = None
        self.isConnected = False
        self.isDisconnected = False


    def run(self):
        """
        starts the socket communication
        :return: none.
        """

        while not self.isConnected:
            self.connect()
        while not self.isDisconnected:
            time.sleep(1)


    def connect(self):
        """
        tries to connect to desierd ip and port, and throws exception if no connection is avalible

        :return:none.
        """
        try:
            self.socket.connect((self.ip, self.port))
            if self.debug:
                print("got connection!")
            self.isConnected = True

        except OSError:
            if self.debug:
                print("no socket available!")
            pass

    def shutdown(self):
        """
        shuts down the connection from the server.

        :return:none
        """
        self.socket.close()
        self.isDisconnected = True

    def sendData(self, data: str):
        """
        sends desired data to the server.

        :param data: desired data to send to server
        :return:none.
        """
        if self.isConnected:
            data = data
            self.socket.sendall(data.encode())
        else:
            if self.debug:
                print("cant send message, no connection established!")
            pass

