import json
import math
import numpy as np
import cv2
import base64
import os
from flask import url_for
debug = True

def lidarDataParser(dataToParse):
    """
    pares the retrieved Lidar data from the udp server to a list with points and ranges and filters out "inf" values.
    :param dataToParse: the unparsed retrieved Lidar data from the UDP.
    :return: returns a list of lists with the points and ranges without "inf" values
    """
    if dataToParse is not None:
        parsedData = str(dataToParse.decode("utf-8"))
        parsedData = json.loads(parsedData)
        lidarRanges = parsedData["Lidar"]["Ranges"]
        lidarAngles = parsedData["Lidar"]["Angles"]
        lidarData = [[], []]
        for r, a in zip(lidarRanges, lidarAngles):
            if r != float("inf"):
                lidarData[0].append(r)
                lidarData[1].append(math.degrees(a))
    else:
        lidarData = [[0] * 360, [0] * 360]

    return lidarData


def vesselInfoDataParser(dataToParse):
    """
    pares the retrieved vessel info data from the udp server into a list of seven lists containing all motor information,
    and Kalman generated vessel velocity.
    :param dataToParse: the retrived UDP Data from the server.
    :return: a list of seven lists containing all motor information,
    and Kalman generated vessel velocity.
    """

    if dataToParse is not None:
        if debug == True:
            print("recived Vessel data: ")
            print(dataToParse)
        parsedData = str(dataToParse.decode("utf-8"))
        parsedData = json.loads(parsedData)
        forwardRightMotor = parsedData["motors"]["forwardRight"]
        turnLeftMotor = parsedData["motors"]["turnLeft"]
        turnRightMotor = parsedData["motors"]["turnRight"]
        forwardLeftMotor = parsedData["motors"]["forwardLeft"]
        vesselXDir = parsedData["kalman"]["speedX"]
        vesselYDir = parsedData["kalman"]["speedY"]
        vesselThetaDir = parsedData["kalman"]["speedTheta"]
        vesselData = [ int(turnLeftMotor), int(forwardRightMotor), int(forwardLeftMotor), int(turnRightMotor), vesselXDir, vesselYDir, vesselThetaDir]

    else:
        vesselData = [0, 0, 0, 0, 0, 0, 0]

    return vesselData


# def mapDataParser(dataToParse, savingPath):
#     if dataToParse is not None:
#         mapMetaData = dataToParse.pop()
#         if type(mapMetaData) is not dict:
#             mapMetaData = {"color": "RGB", "width": 800, "height": 600} #default meta data
#
#         image = np.array(dataToParse)
#         print(image)
#         if image.size == 144000:
#             image = image.reshape((mapMetaData["height"], mapMetaData["width"], 3))
#             cv2.imwrite(savingPath, image)

def mapDataParser(dataToParse):
    """
    pares the map data retrieved from the UDP server.
    :param dataToParse: The map data retrieved form the UDP server.
    :return: returns the decoded image.
    """
    if debug == True:
        print("recived map data data: ")
        print(dataToParse)

    stringData = dataToParse
    restoredImage = None
    if stringData is not None:
        orginalImage = base64.b64decode(stringData)
        pngAsNp = np.frombuffer(orginalImage, dtype=np.uint8)
        restoredImage = cv2.imdecode(pngAsNp, flags=1)
    return(restoredImage)